<?php
/**
 * Plugin Name: GLS Simple RSS Reader
 * Plugin URI: https://gls-inc.co.jp/archives/465
 * Description: Display RSS feed items with thumbnails / サムネイル付きで記事を表示できるRSSリーダーです。
 * Version: 1.0.0
 * Author: GLS Inc.
 * Author URI: https://gls-inc.co.jp/
 * License: GPL v2 or later
 * License URI: https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain: gls-simple-rss-reader
 */

if (!defined('ABSPATH')) {
    exit;
}

$GLOBALS['gls_rss_reader_running'] = false;

add_filter('wp_feed_cache_transient_lifetime', function($lifetime) {
    return $GLOBALS['gls_rss_reader_running'] ? 0 : $lifetime;
});

function gls_rss_reader_shortcode($atts) {
    global $GLOBALS;
    $GLOBALS['gls_rss_reader_running'] = true;

    $atts = shortcode_atts(array(
        'url' => '',
        'items' => 5,
    ), $atts, 'gls_rss_reader');

    if (empty($atts['url'])) {
        $GLOBALS['gls_rss_reader_running'] = false;
        return '<p>No RSS feed URL specified.</p>';
    }

    include_once(ABSPATH . WPINC . '/feed.php');
    $rss = fetch_feed($atts['url']);
    if (is_wp_error($rss)) {
        $GLOBALS['gls_rss_reader_running'] = false;
        return '<p>Failed to fetch RSS feed.</p>';
    }

    $maxitems = $rss->get_item_quantity((int)$atts['items']);
    $rss_items = $rss->get_items(0, $maxitems);

    $output = '<div class="gls-rss-reader">';

    foreach ($rss_items as $item) {
        $title = esc_html($item->get_title());
        $link = esc_url($item->get_permalink());
        $description_raw = $item->get_description();

        $img_url = '';
        preg_match('/<img[^>]+src="([^"]+)"/i', $description_raw, $img_match);
        if (!empty($img_match[1])) {
            $img_url = esc_url($img_match[1]);
        } else {
            $content_encoded = $item->get_item_tags('http://purl.org/rss/1.0/modules/content/', 'encoded');
            if (!empty($content_encoded[0]['data'])) {
                preg_match('/<img[^>]+src="([^"]+)"/i', $content_encoded[0]['data'], $content_img_match);
                if (!empty($content_img_match[1])) {
                    $img_url = esc_url($content_img_match[1]);
                }
            }
        }

        $desc_text = wp_strip_all_tags($description_raw);
        $desc_text = trim($desc_text);

        if (!empty($desc_text)) {
            $description_text = mb_substr($desc_text, 0, 120) . '...';
        } else {
            $content_encoded = $item->get_item_tags('http://purl.org/rss/1.0/modules/content/', 'encoded');
            $content_text = '';
            if (!empty($content_encoded[0]['data'])) {
                $content_text = wp_strip_all_tags($content_encoded[0]['data']);
                $content_text = trim($content_text);
            }
            $description_text = mb_substr($content_text, 0, 120) . '...';
        }

        $pubDate = esc_html($item->get_date('Y.m.d'));

        $output .= '<a href="' . $link . '" class="gls-rss-item" target="_blank" rel="noopener">';
        $output .= '<div class="gls-rss-thumbnail">';
        if (!empty($img_url)) {
            $output .= '<img src="' . $img_url . '" alt="RSS thumbnail" />';
        }
        $output .= '</div>';
        $output .= '<div class="gls-rss-content">';
        $output .= '<div class="gls-rss-title">' . $title . '</div>';
        $output .= '<div class="gls-rss-description">' . esc_html($description_text) . '</div>';
        $output .= '<div class="gls-rss-date">' . $pubDate . '</div>';
        $output .= '</div></a>';
    }

    $output .= '</div>';

    $GLOBALS['gls_rss_reader_running'] = false;
    return $output;
}
add_shortcode('gls_rss_reader', 'gls_rss_reader_shortcode');

function gls_rss_reader_styles() {
    echo '<style>
    .gls-rss-reader {
        display: flex;
        flex-direction: column;
        gap: 16px;
    }
    .gls-rss-item {
        display: flex;
        align-items: flex-start;
        text-decoration: none;
        background: #fff;
        border: 1px solid #ddd;
        padding: 12px;
        transition: box-shadow 0.3s ease;
        border-radius: 8px;
    }
    .gls-rss-item:hover {
        box-shadow: 0 4px 12px rgba(0,0,0,0.15);
    }
    .gls-rss-thumbnail {
        flex: 0 0 120px;
        height: 80px;
        background: #eee;
        display: flex;
        align-items: center;
        justify-content: center;
        overflow: hidden;
        border-radius: 4px;
        margin-right: 12px;
    }
    .gls-rss-thumbnail img {
        max-width: 100%;
        height: auto;
    }
    .gls-rss-content {
        flex: 1;
    }
    .gls-rss-title {
        font-weight: bold;
        color: #333;
        margin-bottom: 6px;
        font-size: 16px;
    }
    .gls-rss-description {
        color: #555;
        font-size: 14px;
        margin-bottom: 4px;
    }
    .gls-rss-date {
        color: #999;
        font-size: 12px;
    }
    </style>';
}
add_action('wp_head', 'gls_rss_reader_styles');

// 設定画面
function gls_rss_reader_add_settings_page() {
    add_options_page(
        'GLS Simple RSS Reader Settings',
        'GLS Simple RSS Reader',
        'manage_options',
        'gls-simple-rss-reader',
        'gls_rss_reader_settings_page'
    );
}
add_action('admin_menu', 'gls_rss_reader_add_settings_page');

function gls_rss_reader_settings_page() {
    echo '<div class="wrap">';
    echo '<h1>GLS Simple RSS Reader</h1>';
    echo '<h2>How to use the shortcode / ショートコードの使い方:</h2>';
    echo '<pre>[gls_rss_reader url="Feed URL" items="Number"]</pre>';
    echo '<p>Example / 例:</p>';
    echo '<pre>[gls_rss_reader url="https://example.com/feed" items="3"]</pre>';
    echo '</div>';
}

// プラグインページに「設定」リンク追加
function gls_rss_reader_plugin_action_links($links) {
    $settings_link = '<a href="options-general.php?page=gls-simple-rss-reader">' . __('Settings', 'gls-simple-rss-reader') . '</a>';
    array_unshift($links, $settings_link);
    return $links;
}
add_filter('plugin_action_links_' . plugin_basename(__FILE__), 'gls_rss_reader_plugin_action_links');
